import React, { useState, useEffect } from "react";
import {
  Box,
  List,
  ListItem,
  ListItemText,
  ListItemIcon,
  IconButton,
  Tooltip,
} from "@mui/material";
import {
  Folder as FolderIcon,
  Psychology as LLMIcon,
  ImageSearch as OCRIcon,
  Search as SearchIcon,
} from "@mui/icons-material";
import { DocumentMetadata } from "../../../electron/types";
import DirectoryPicker from "./DirectoryPicker";
import { useFileSystem } from "../../hooks/useFileSystem";
import DirectorySearchModal from "../DirectorySearchModal";

interface FileMetadata {
  [path: string]: DocumentMetadata;
}

interface Directory {
  name: string;
  path: string;
}

export default function FileExplorer() {
  const { watchedDirectories, watchDirectory, unwatchDirectory } = useFileSystem();
  const [fileMetadata, setFileMetadata] = useState<FileMetadata>({});
  const [openModal, setOpenModal] = useState<{ [key: string]: boolean }>({});

  useEffect(() => {
    window.electron.getDocuments().then((response) => {
      if (response.success && response.data) {
        const metadata: FileMetadata = {};
        response.data.forEach((doc) => {
          metadata[doc.path] = doc;
        });
        setFileMetadata(metadata);
      }
    });
  }, [watchedDirectories]);

  const handleDirectorySelect = async (dirPath: string) => {
    if (!watchedDirectories.includes(dirPath)) {
      await watchDirectory(dirPath);
    }
  };

  const handleDirectoryRemove = async (dirPath: string) => {
    await unwatchDirectory(dirPath);
  };

  const handleOpenModal = (dir: string) => {
    setOpenModal((prev) => ({ ...prev, [dir]: true }));
  };

  const handleCloseModal = (dir: string) => {
    setOpenModal((prev) => ({ ...prev, [dir]: false }));
  };

  const directoryList: Directory[] = watchedDirectories.map((dir) => ({
    name: dir.split(/[/\\]/).pop() || dir,
    path: dir,
  }));

  return (
    <Box
      sx={{
        display: "flex",
        flexDirection: "column",
        height: "100%",
        overflow: "hidden",
      }}
    >
      <DirectoryPicker directories={directoryList} onSelect={handleDirectorySelect} />

      <Box sx={{ flex: 1, overflow: "auto" }}>
        <List>
          {watchedDirectories.map((dir) => {
            const modalId = `modal-${dir}`;

            return (
              <ListItem key={dir} sx={{ display: "flex", alignItems: "center" }}>
                <ListItemIcon sx={{ minWidth: 28 }}>
                  <FolderIcon fontSize="small" />
                </ListItemIcon>

                {fileMetadata[dir]?.hasEmbeddings && (
                  <Tooltip title="LLM Embeddings Enabled">
                    <LLMIcon color="primary" fontSize="small" />
                  </Tooltip>
                )}
                {fileMetadata[dir]?.hasOcr && (
                  <Tooltip title="OCR Enabled">
                    <OCRIcon color="secondary" fontSize="small" />
                  </Tooltip>
                )}

                <ListItemText
                  primary={dir.split(/[/\\]/).pop()}
                  secondary={dir}
                  secondaryTypographyProps={{
                    sx: {
                      overflow: "hidden",
                      textOverflow: "ellipsis",
                      whiteSpace: "nowrap",
                    },
                  }}
                />

                {/* Search Icon to Open Modal */}
                <Tooltip title="Search">
                  <IconButton color="primary" size="small" onClick={() => handleOpenModal(dir)}>
                    <SearchIcon fontSize="small" />
                  </IconButton>
                </Tooltip>

                <DirectorySearchModal
                  dir={dir}
                  open={!!openModal[dir]}
                  onClose={() => handleCloseModal(dir)}
                  modalId={modalId}
                />
              </ListItem>
            );
          })}
        </List>
      </Box>
    </Box>
  );
}
